#!/usr/bin/perl -w

use CGI 'header', 'param';
use CGI::Carp 'fatalsToBrowser';
$CGI::POST_MAX=1024 * 500;  # max 500K posts
$CGI::DISABLE_UPLOADS = 1;  # no uploads 
$query = new CGI;	# Setup CGI object

##
## quickmblog (Quick Multiuser Blogger)
## Written by Aaron Hachez, 2003
## Released under GPL (Gnu General Public License)    
## See included file LICENSE.txt for more details, or
## Visit http://www.gnu.org for a copy.
##
## show.pl
## 
## Purpose: To read a given range of files from a list of files which contain individual entries
## and have a naming convention of <year>.<month>.<day>.<hour>.<minute>.<second>.quickmblog
## Range may include quantity i.e. 20 thru 50.
##

####################
### Beg Varibles ###

# The allowed_hosts and denied_hosts use the following syntax
#
# You must use ip addresses, excepting a single "*" which indicates
# match all.  You can use a partial IP address (i.e. 10.0., or 10.0.0.),
# or a "*" to match all. Allow is checked first, if the IP is in the
# allow list, or allow is set to it's default "*", no further checking is done.
# if an IP is not in the allow list, the deny list is checked next.
# If the IP is in neither list, and neither list is using the "*" wildcard,
# the user is let through.
#
# In this example, only users coming from addresses begining
# with "10.0." or "192.168.0." are allowed through, and all others are denied.
#
# @allowed_hosts = ("10.0.", "192.168.0.");
# @denied_hosts = ("*");
#
# In this example, allow is set to match all, so everyone is allowed, and the deny
# list is not checked.
#
# @allowed_hosts = ("*");
# @denied_hosts = ("10.0.0.", "192.168.");
#
# In this example, allow is set to nothing, so only those addresses listed in
# the deny list are refused. Setting the deny list to nothing is effectively
# the same as setting the allow to "*".
#
# @allowed_hosts = ("");
# @denied_hosts = ("10.0.0.", "192.168.");

# An allowed hosts list
@allowed_hosts = ("*");

# An denied hosts list
@denied_hosts = ("*");

#########################################
## You probably don't need to change   ##
## Anything below this point, but can. ##
#########################################

# Myself
$self = "show.pl";

# The add script
$add_script = "add.pl";

# The edit script
$edit_script = "edit.pl";

# The delete script
$delete_script = "delete.pl";

# This is used for printing out the edit/delete form
# Note, in order for this to work properly,
# a tag must be included for the value of filename "<!--$FILE$-->"
# for both edit and delete entries.
# This tag is replaced with the name of the current file loading
# and is used later in editing/deleting said file
# the $edit_script and $delete_script variables are also important.
# NOTE: Do not include newlines "\n", as they will cause the 
# function to fail.
$edit_delete_form = "<table width=\"100\%\" border=\"0\">";
$edit_delete_form .= "<tr><td align=\"right\" valign=\"center\">";

# This is the html form, comment this, and uncomment the next 13 lines for edit and delete buttons instead
$edit_delete_form .= "<a href=\"$add_script\">add new</a> | <a href=\"$edit_script?filename=<\!--\$FILE\$-->\">edit</a> | <a href=\"$delete_script?filename=<\!--\$FILE\$-->\">delete</>";

# $edit_delete_form .= "<form action=\"$add_script\" method=\"post\">";
# $edit_delete_form .= "<input name=\"add\" type=\"submit\" value=\"Add New\">";
# $edit_delete_form .= "</form>";
# $edit_delete_form .= " | ";
# $edit_delete_form .= "<form action=\"$edit_script\" method=\"post\">";
# $edit_delete_form .= "<input name=\"filename\" type=\"hidden\" value=\"<\!--\$FILE\$-->\">";
# $edit_delete_form .= "<input name=\"edit\" type=\"submit\" value=\"Edit\">";
# $edit_delete_form .= "</form>";
# $edit_delete_form .= " | ";
# $edit_delete_form .= "<form action=\"$delete_script\" method=\"post\">";
# $edit_delete_form .= "<input name=\"filename\" type=\"hidden\" value=\"<\!--\$FILE\$-->\">";
# $edit_delete_form .= "<input name=\"delete\" type=\"submit\" value=\"Delete\">";
# $edit_delete_form .= "</form>";

$edit_delete_form .= "</td></tr></table>";

# The path to user entries
$entries_path = "./entries/";

# The path to display templates
$template_path = "./templates/";

$template_file = "show.html";

# Entry seperator
$seperator = "<hr noshade>";

# Default limit for displaying entries
$default_limit = 50;

### End Varibles ###
####################
################
### BEG MAIN ###

# Obligitory content type statment
print "Content-type: text/html\n\n";

# Initially set allowed to false
my($allowed) = 0;

# Before we do anything else, check the remote host(s)
# Check first to see if allowed, if not in the allowed list, Check the denied list.
# If they are in the allow list
if(check_allowed_hosts($query->remote_addr()))
{
	$allowed = 1;
}
# If they are _not_ in the denied list
elsif(!check_denied_hosts($query->remote_addr()))
{
	$allowed = 1;
}

# If the caller was either in the allow list, or not in the deny list
if($allowed eq 1)
{
	my($start) = param('start');
	my($limit) = param('limit');
	my($search_date) = param('search_date');
	my($search_user) = param('search_user');
	my($search_term) = param('search_term');

	# If start is not defined
	if(!defined($start))
	{
		$start = 0;
	}

	# If limit is not defined
	if(!defined($limit))
	{
		# Limit equals the default limit
		$limit = $default_limit;
	}

	# Check to see if next or prev has been clicked
	if(param('next'))
	{
		$start += $limit;
	}
	elsif(param('prev'))
	{
		$start -= $limit;
	}

	# Make sure that we don't try to work with negative numbers
	if($start < 0)
	{
		$start = 0;
	}

	if($limit < 0)
	{	
		$limit = 0;
	}

	# Load the template file
	@template = get_template($template_file);

	# For each line of the template file
	foreach $arg (@template)
	{
		# Find the line that looks like <!--$ENTRY$-->
		# and replace it with the specified number of entries
		if($arg =~ /^(.*)\<\!--\$ENTRY\$--\>(.*)$/)
		{
			# Store data before and after entry
			my($before) = $1;
			my($after) = $2;
		
			# Get a sorted list of the files in the entries dir
			my(@filelist) = get_sorted_file_list();

			# If a date search has been requested
			if(defined($search_date))
			{
				if($search_date ne "")
				{
					# Debug
					#print "Search date: $search_date\n";
 
					# Search for all files matching date
					@filelist = search_entries_date($search_date, scalar(@filelist), @filelist);
				}
			}
			
			# If a user search has been requested
			if(defined($search_user))
			{
				if($search_user ne "")
				{
					# Debug
					#print "Search user: $search_user\n";
 
					# Search for all files matching date
					@filelist = search_entries_user($search_user, scalar(@filelist), @filelist);
				}
			}
			
			# If a specific term search has been requested
			if(defined($search_term))
			{
				if($search_term ne "")
				{
					# Debug
					#print "Search term: $search_term\n";
 
					# Search for all files matching term
					@filelist = search_entries_term($search_term, scalar(@filelist), @filelist);
				}
			}

			# Get total number of files
			my($max) = scalar(@filelist);

			# If first requested file is not the "first" one
			if($start > 0)
			{
				# Chop off elements up to the first requested.
				splice(@filelist, 0, $start);
			}

			# If the first postion plus limit is less than the max
			if(($start + $limit) < $max)
			{
				# Chop off the elements after the first+limit
				splice(@filelist, ($start + $limit) - $max);
			}
	
			print $before; # Print whatever was before the entry on the line
			get_entries(@filelist);
			print $after; # Print whatever was after the entry on the line
		}
		else
		{
			# Replace various specific entries
			$arg =~ s/^(.*)\<\!--\$ADD\$--\>(.*)$/$1$add_script$2/g;
			$arg =~ s/^(.*)\<\!--\$ACTION\$--\>(.*)$/$1$self$2/g;
			$arg =~ s/^(.*)\<\!--\$START\$--\>(.*)$/$1$start$2/g;
			$arg =~ s/^(.*)\<\!--\$LIMIT\$--\>(.*)$/$1$limit$2/g;
		
			if(defined($search_user))
			{	
				$arg =~ s/^(.*)\<\!--\$SEARCH-USER\$--\>(.*)$/$1$search_user$2/g;
			}
			else
			{
				$arg =~ s/^(.*)\<\!--\$SEARCH-USER\$--\>(.*)$/$1$2/g;
			}

			if(defined($search_date))
			{	
				$arg =~ s/^(.*)\<\!--\$SEARCH-DATE\$--\>(.*)$/$1$search_date$2/g;
			}
			else
			{
				$arg =~ s/^(.*)\<\!--\$SEARCH-DATE\$--\>(.*)$/$1$2/g;
			}
			
			if(defined($search_term))
			{	
				$arg =~ s/^(.*)\<\!--\$SEARCH-TERM\$--\>(.*)$/$1$search_term$2/g;
			}
			else
			{
				$arg =~ s/^(.*)\<\!--\$SEARCH-TERM\$--\>(.*)$/$1$2/g;
			}

			print $arg;
		}
	}
}
# Else caller is not allowed to access
else
{
	print "You have been denied access.\n";
}

### END MAIN ###
################
#####################
### Sub Functions ###

## Get the template file for showing entries
# takes one arg, filename
sub get_template
{
	if(!$_[0])
	{
		die "No filename passed for template.\n";
	}

	# Get the file name and path from first arg passed
	my($filename) = $_[0];
	
	# Open template file for read
	open(TEMPLATE, "$template_path/$filename") || die "Couldn't find template file: $filename\n";

	# Get entire template file contents
	my(@template) = <TEMPLATE>;

	# Close template file
	close(TEMPLATE);

	# Return template file contents
	return(@template);
}

sub get_sorted_file_list
{
	my(@filelist);			# A list of files in the entries dir
	my(@sorted_filelist);	# The same list, sorted
	my($max) = 0;			# The total number of entries
	
	# Open entries directory so that we can read in file list
	opendir(DIR, "$entries_path") || die "Couldn't open entries directory: $entries_path\n";
	
	## While not at the end of the dir list
	while(defined($arg = readdir(DIR)))
	{
		## If $arg = . or .. don't push it onto the array
		if (!(($arg =~ /^\.$/) || ($arg =~ /^\.\.$/)))
		{
			unshift @filelist, $arg;
			++$max;
		}
	}

	closedir(DIR);

	# Sort list numerically desceding, reverse the position of a and b to sort ascending
	@sorted_filelist = sort  {$b cmp $a} @filelist;

	return(@sorted_filelist);
}

# Search for entries matching date
# Returns a list of filenames
sub search_entries_date
{
	my(@new_filelist);	# The filelist to be returned
	my($i) = 0;			# A counter

	# If more than three elements have been passed
	if(scalar(@_) > 2)
	{
		# Here we shift of the first 2 elements, and assign them to their own variables.
		my($date) = shift(@_);		# The date requested
		my($num_files) = shift(@_);	# The number of files passed
		my(@filelist) = @_;			# The file list

		# Check to insure the proper number of files were passed
		if(scalar(@filelist) ne $num_files)
		{
			print "Wrong number of filenames passed. $num_files " .  scalar(@filelist) . "\n";
		}
		# Else, go ahead and process search.
		else
		{
			# Make sure the date is in the proper format, "" or YYYY.MM.DD
			if($date =~ /[0-9][0-9][0-9][0-9]\.[0-9][0-9]\.[0-9][0-9]/)
			{
				# For each file in the file list
				foreach $file (@filelist)
				{
					# Look for the date specified
					# This is going to be matched in the filename
					if($file =~ /^$date.*$/)
					{
						# Add this file to the list of files
						$new_filelist[$i] = $file;

						# Increment counter
						++$i;
					}
				}
			}
			else
			{
				print "Bad date format: $date, should be in the form: YYYY.MM.DD\n";
			}
		}
	}

	return(@new_filelist);
}

# Search for entries matching user
# Returns a list of filenames
sub search_entries_user
{
	my(@new_filelist);	# The filelist to be returned
	my($i) = 0;			# A counter

	# If more than three elements have been passed
	if(scalar(@_) > 2)
	{
		# Here we shift of the first 2 elements, and assign them to their own variables.
		my($user) = shift(@_);		# The user requested
		my($num_files) = shift(@_);	# The number of files passed
		my(@filelist) = @_;			# The file list

		# Check to insure the proper number of files were passed
		if(scalar(@filelist) ne $num_files)
		{
			print "Wrong number of filenames passed. $num_files " .  scalar(@filelist) . "\n";
		}
		# Else, go ahead and process search.
		else
		{
			# Make sure the user is in the proper format, not ""
			if($user ne "")
			{
				# For each file in the file list
				foreach $file (@filelist)
				{
					# Look for the date specified
					# This is going to be matched in the filename
					if($file =~ /^.*\.$user$/)
					{
						# Add this file to the list of files
						$new_filelist[$i] = $file;

						# Increment counter
						++$i;
					}
				}
			}
			else
			{
				print "Bad user format: $user\n";
			}
		}
	}

	return(@new_filelist);
}

# Search for entries matching term
# Returns a list of filenames
sub search_entries_term
{
	my(@new_filelist);	# The filelist to be returned
	my($i) = 0;			# A counter

	# If more than three elements have been passed
	if(scalar(@_) > 2)
	{
		# Here we shift of the first 2 elements, and assign them to their own variables.
		my($term) = shift(@_);		# The term requested
		my($num_files) = shift(@_);	# The number of files passed
		my(@filelist) = @_;			# The file list

		# Check to insure the proper number of files were passed
		if(scalar(@filelist) ne $num_files)
		{
			print "Wrong number of filenames passed. $num_files " .  scalar(@filelist) . "\n";
		}
		# Else, go ahead and process search.
		else
		{
			# Make sure the term is in the proper format, not ""
			if($term ne "")
			{
				# For each file in the file list
				foreach $file (@filelist)
				{
					open(FILE, "$entries_path/$file") || die "Couldn't open file for read: $file\n";

					foreach $arg (<FILE>)
					{
						if($arg =~ /^.*$term.*$/)
						#if(index($arg,$term) >= 0)
						{
							# Add this file to the list of files
							$new_filelist[$i] = $file;

							# Increment counter
							++$i;

							# We only need to match once, so exit loop
							last;
						}
					}

					close(FILE);
				}
			}
			else
			{
				print "Bad search term. $term\n";
			}
		}
	}

	return(@new_filelist);
}

# Load and print n entry files from the entries directory starting with i
# takes two args, start, limit
sub get_entries
{
	my(@filelist) = @_;			# A list of files in the entries dir

	if(defined(@filelist))
	{
		foreach $filename (@filelist)
		{
			# Print the seperator, specified in the top of this file
			print "$seperator\n";

			# Open entry file numbered $start
			open(FILE, "$entries_path/$filename") || die "Couldn't open file: $entries_path/$filename\n";

			# For debug
			#print "$entries_path/$filename\n";
	
			## Used to record the number of lines
			my($num_lines) = 0;

			foreach $arg (<FILE>)
			{
				# This cuts off the first two lines of each entry.
				# The first two lines contain data about who posted this entry,
				# and when they made edits.
				if($num_lines >= 1)
				{
					$arg =~ s/\n/\<br\>\n/g;
					print "$arg";
				}

				++$num_lines;
			}

			close(FILE);

			# If edit_delete_form contains our special file tag
			# take out the tag, and replace with the current filename
			my($output_form) = $edit_delete_form;
			if($output_form =~ s/\<\!--\$FILE\$--\>/$filename/g)
			{
				print $output_form;
			}
		}
	}
	else
	{
		print $seperator;
		print "No entries found.";
	}

	# Print the seperator, specified in the top of this file
	print "$seperator\n";
}

# Check the @allowed_hosts list to see if current caller is allowed
sub check_allowed_hosts
{
	my($result) = 0;

	# If an argument has been passed
	if(defined($_[0]))
	{
		# Get arg in local variable
		my($remote_host) = $_[0];

		# Debug
		#print "Allowedlist: @allowed_hosts, remotehost: $remote_host\n";

		# If our allowed hosts list first element eq a single "*" indicating all
		if($allowed_hosts[0] eq "*")
		{
			$result = 1;
		
			# Debug
			#print "ALLOWED BY *";
		}
		else
		{
			# Loop through the allowed list
			foreach $arg (@allowed_hosts)
			{
				# If arg is not blank
				if($arg ne "")
				{
					# If the remote matches 
					if($remote_host =~ /^$arg.*$/)
					{
						# Debug
						#print "ALLOWED BY MATCH";

						$result = 1;
						last;
					}
				}
			}
		}
	}

	return($result);
}

# Check the @denied_hosts list to see if current caller is denied
sub check_denied_hosts
{
	my($result) = 0;

	# If an argument has been passed
	if(defined($_[0]))
	{
		# Get arg in local variable
		my($remote_host) = $_[0];

		# Debug
		#print "Deniedlist: @denied_hosts, remotehost: $remote_host\n";

		# If our denied hosts list first element eq a single "*" indicating all
		if($denied_hosts[0] eq "*")
		{
			$result = 1;
			
			# Debug
			#print "DENIED BY *";
		}
		else
		{
			# Loop through the allowed list
			foreach $arg (@denied_hosts)
			{
				# If arg is not blank
				if($arg ne "")
				{
					# If the remote matches 
					if($remote_host =~ /^$arg.*$/)
					{
						# Debug
						#print "DENIED BY MATCH";

						$result = 1;
						last;
					}
				}
			}
		}
	}

	return($result);
}
